require ("math")

function love.load()
	screenWidth = 600
    screenHeight = 600
	love.window.setMode(screenWidth,screenHeight)
	love.graphics.setBackgroundColor(0,180,200)

	meter = (64) -- our scale for the physics world

	world = love.physics.newWorld(0, 9.81*meter, true)
	world:setCallbacks(beginContact, endContact, preSolve, postSolve)

    rng = love.math.newRandomGenerator()
    rng:setSeed(os.time())

	objects = {}
	--re-useable colors
	red    = {200,20 ,20 }
	green  = {20 ,200,20 }
	blue   = {20 ,20 ,200}
	yellow = {200,200,20 }
	orange = {230,110,20 }
	pink   = {200,20 ,200}
	cyan   = {20 ,200,200}
	numberOfColors = 7
	numberOfShapes = 1

	grey   = {150,150,150}
	glowing= {0  ,0  ,0  }
	glowIntensity = 0
	runTime = 0

	blockShape = love.physics.newRectangleShape(0, 0, 50, 50)
	circleShape = love.physics.newCircleShape(30)
	bigBlockShape = love.physics.newRectangleShape(0, 0, 65, 65)
	wideBlockShape = love.physics.newRectangleShape(0, 0, 100, 50)
	tallBlockShape = love.physics.newRectangleShape(0, 0, 50, 100)


	nextColor = randomColor(numberOfColors)
	print("nextColor: ")
	print(nextColor)
	nextShape = randomShape(numberOfShapes)
	print("nextShape: ")
	print(nextShape)

	timescale = 1/8

	score = 0
	lostObjects = 0
	lostLimit = 6
	lostObjects = 2
	gameover = false

	totalGain = 0
	totalLoss = 0

	addLocation = {}
	addLocation.x = 0
	addLocation.y = 530

	--let's create the ground :D :D :D :D
	ground = {}
	ground.body = love.physics.newBody(world, screenWidth/2, screenHeight) --physics bodies orient by their center
	ground.shape = love.physics.newRectangleShape(800, 250)
	ground.fixture = love.physics.newFixture(ground.body, ground.shape) --attach shape to body
	ground.fixture:setUserData("ground") --differentiate ground from others

		ground2 = {}
		ground2.body = love.physics.newBody(world, (screenWidth/2)+250, screenHeight+100) --physics bodies orient by their center
		ground2.shape = love.physics.newRectangleShape(100, 750)
		--ground2.body:setAngle(0.05)
		ground2.fixture = love.physics.newFixture(ground2.body, ground2.shape) --attach shape to body
		ground2.fixture:setFriction(0)
		ground2.fixture:setUserData("ground") --differentiate ground from others
		ground3 = {}
		ground3.body = love.physics.newBody(world, (screenWidth/2)-250, screenHeight+100) --physics bodies orient by their center
		ground3.shape = love.physics.newRectangleShape(100, 750)
		--ground3.body:setAngle(-0.05)
		ground3.fixture = love.physics.newFixture(ground3.body, ground3.shape) --attach shape to body
		ground3.fixture:setFriction(0)
		ground3.fixture:setUserData("ground") --differentiate ground from others

	sound1 = love.audio.newSource("good2.wav", "static") --score up
	sound2 = love.audio.newSource("bad.wav",   "static") --lost block
	sound3 = love.audio.newSource("blip.wav",  "static") --place block
	sound4 = love.audio.newSource("ded.wav",   "static") --game over
	sound5 = love.audio.newSource("zap.wav",   "static") --glowy noise

	myfont = love.graphics.newFont( 40 )
	love.graphics.setFont(myfont)

	--start the game with blocks on-screen:
	w = 6
	h = 4
	for x = 326-(w*26), 274+(w*26), 52 do --6 blocks wide
		for y = 400-(h*50),399, 50 do --2 blocks high
			previousColor = nextColor
			while previousColor == nextColor do
				nextColor = randomColor(numberOfColors)
			end
			addObject(x,y,nextColor, blockShape)
		end
	end
end

function addObject(x,y,color,shape)
	local color = color
	local shape = shape
	newid = table.getn(objects)+1
	objects[newid] = {}
	objects[newid].body = love.physics.newBody(world, x, y, "dynamic")
	objects[newid].shape = shape
	objects[newid].fixture = love.physics.newFixture(objects[newid].body, objects[newid].shape,5)
	objects[newid].color = color
	objects[newid].matching = 0
	objects[newid].dying = -1
	objects[newid].fixture:setRestitution(0.3)
	objects[newid].fixture:setFriction(0.2)
	objects[newid].fixture:setUserData(newid) -- used in collision callbacks so each object can have a corresponding position in this array
end

function love.update(dt)
	world:update(dt*timescale)
	world:update(dt*timescale)
	world:update(dt*timescale)
	world:update(dt*timescale)
	world:update(dt*timescale)
	world:update(dt*timescale)
	world:update(dt*timescale)
	world:update(dt*timescale)

	addLocation.x = love.mouse.getX()
	if addLocation.x < (screenWidth/2)-200 then addLocation.x = (screenWidth/2)-200 end
	if addLocation.x > (screenWidth/2)+200 then addLocation.x = (screenWidth/2)+200 end

	for i, obj in pairs(objects) do
		if obj ~= nil then
			if obj.dying > -1 then
				objects[i].dying = objects[i].dying + dt -- Objects don't clear until after a timer is up
				if obj.dying > 1.75 then
					objects[i].body:destroy()
					objects[i] = nil
					if gameover == false then
						score = score + 100
						totalGain = totalGain + 100
						print("gain:"..totalGain)
						lostObjects = lostObjects - 0.05
						sound1:stop()
						sound1:play()
					end
					if lostObjects < 0 then lostObjects = 0 end
				end
			else
				x, y = objects[i].body:getPosition( ) -- Objects that fall off without being matched are counted
				if x > ( (screenWidth/2)+250) or x < ( (screenWidth/2)-250) then
					objects[i].body:destroy()
					objects[i] = nil
					if gameover == false then
						lostObjects = lostObjects + 1
						score = score - 100
						totalLoss = totalLoss + 100
						sound2:stop()
						sound2:play()
						print("loss:"..totalLoss)
					end
				end
			end
		end
	end
	if lostObjects >= lostLimit and gameover == false then
		gameover = true
		sound4:play()
	end --game ends when too many blocks fall off the screen

	


	runTime = runTime + dt
	glowIntensity = 205 + (50*math.sin(runTime*6))	--make stuff pulsate :D
	--print(glowIntensity)
	glowing = {glowIntensity, glowIntensity, glowIntensity}
end

function love.draw()
	love.graphics.setLineWidth(3)
	love.graphics.setColor(50,50,50) -- set the drawing color to for the ground
	love.graphics.polygon("fill", ground.body:getWorldPoints(ground.shape:getPoints())) -- draw a "filled in" polygon using the ground's coordinates
	love.graphics.polygon("fill", ground2.body:getWorldPoints(ground2.shape:getPoints()))
	love.graphics.polygon("fill", ground3.body:getWorldPoints(ground3.shape:getPoints()))
	love.graphics.setColor(red)
	love.graphics.rectangle("fill",0,580,screenWidth-(screenWidth*(lostObjects/lostLimit)),20) --Basically a health bar

	for i, obj in pairs(objects) do
		if obj ~= nil then
			local shapeType = obj.shape:getType()
			love.graphics.setColor(obj.color)
			if shapeType == "circle" then
				love.graphics.circle("fill", obj.body:getX(), obj.body:getY(), obj.shape:getRadius())
				if obj.dying > -1 then love.graphics.setColor(glowing) else love.graphics.setColor(0,0,0) end
				love.graphics.circle("line", obj.body:getX(), obj.body:getY(), obj.shape:getRadius())
			end

			if shapeType == "polygon" then
				love.graphics.polygon("fill", obj.body:getWorldPoints(obj.shape:getPoints()))
				if obj.dying > -1 then love.graphics.setColor(glowing) else love.graphics.setColor(0,0,0) end --color text based on whether the block has been matched
				love.graphics.polygon("line", obj.body:getWorldPoints(obj.shape:getPoints()))
				--love.graphics.print(obj.matching, obj.body:getWorldCenter())-- would print the number of touching shapes, on the shape
			end
		end
	end

	drawPreviewShape()

	if gameover then
		love.graphics.setColor(glowing)
		love.graphics.print("Game over, man!", 130, 200, 0)
		love.graphics.setColor(255,255,255)
		love.graphics.print("SCORE:"..score, 200, 270, 0)--draw score in different location on gameover
		print("gain:"..totalGain)
		print("loss:"..totalLoss)
	else
		love.graphics.setColor(255,255,255)
		love.graphics.print("SCORE:"..score, 300, 20, 0)
	end
end

function love.mousepressed(x,y,button)
	if gameover == false then
		addObject(addLocation.x,530,nextColor,nextShape)--place the shape and THEN pick a new random shape
		nextColor = randomColor(numberOfColors)
		nextShape = randomShape(numberOfShapes)
		sound3:stop()
		sound3:play()
	end
end

function beginContact(a,b,coll)
	local dataA = a:getUserData()
	local dataB = b:getUserData()
	if dataA ~= "ground" and dataB ~= "ground" then
		if objects[dataA].color == objects[dataB].color then
			objects[dataA].matching = objects[dataA].matching + 1
			objects[dataB].matching = objects[dataB].matching + 1
		end
	end
end

function endContact(a,b,coll)
	local dataA = a:getUserData()
	local dataB = b:getUserData()
	if dataA ~= "ground" and dataB ~= "ground" then
		if objects[dataA].color == objects[dataB].color then
			objects[dataA].matching = objects[dataA].matching - 1
			objects[dataB].matching = objects[dataB].matching - 1
		end
	end
end

function postSolve(a, b, coll)
	local dataA = a:getUserData()
	local dataB = b:getUserData()
	local sound = false
	if dataA ~= "ground" and dataB ~= "ground" then
		if objects[dataA].color == objects[dataB].color then
			if objects[dataA].matching > 1 or objects[dataB].matching > 1 or objects[dataA].dying > -1 or objects[dataB].dying > -1 then --Clear groups of 3 or more
				if objects[dataA].dying == -1 then
					objects[dataA].dying = 0
					sound = true
				end
				if objects[dataB].dying == -1 then
					objects[dataB].dying = 0
					sound = true
				end
			end
		end
	end
	if sound then 
		sound5:stop()
		sound5:play() end
end

function love.keypressed(key)

end

function randomColor(number)
	local a = {}
	local high = number - 1
	local n = math.floor( rng:random(0,high) )
	if n < 1 then a = red
	elseif n < 2 then a = green
	elseif n < 3 then a = blue
	elseif n < 4 then a = yellow
	elseif n < 5 then a = pink
	elseif n < 6 then a = cyan
	elseif n <= 7 then a = orange end
	return a
end

function randomShape(number)
	local high = number+3
	local n = ( rng:random(0,high) )
	local a = blockShape
	if n > 4 then a = circleShape end
	if n > 5.5 then a = bigBlockShape end
	return a
end

function drawPreviewShape()
	love.graphics.setColor(nextColor)
	if nextShape == blockShape then
		love.graphics.rectangle("fill", addLocation.x-25, addLocation.y-25, 50,50)
		love.graphics.setColor(0,0,0)
		love.graphics.rectangle("line", addLocation.x-25, addLocation.y-25, 50,50)
	end
	if nextShape == bigBlockShape then
		love.graphics.rectangle("fill", addLocation.x-32.5, addLocation.y-32.5, 65,65)
	end
	if nextShape == wideBlockShape then
		love.graphics.rectangle("fill", addLocation.x-50, addLocation.y-25, 100,50)
	end
	if nextShape == tallBlockShape then
		love.graphics.rectangle("fill", addLocation.x-25, addLocation.y-50, 50,100)
	end

	if nextShape == circleShape then
		love.graphics.circle("fill", addLocation.x, addLocation.y,30)
	end
end